#!/usr/bin/perl -w
# autobull07.pl
# Auteur: Jean-Marie Pacquet
#
# il met  jour la base de donnes (cotations07) du jeu "The Bull"
#  partir de la dernire date de cotations enregistre ou d'une date donne
# jusqu' la date du jour (ou au maximum 1 mois aprs la premire date)
#
use Getopt::Std;
use Date::Calc qw(Today Delta_Days Add_Delta_Days);
# use Crypt::SSLeay;
use URI;
use LWP;
use HTTP::Cookies;
use HTTP::Request::Common qw(POST);
use HTML::Entities ();
use DBI;
use strict;

###############################################################################
#
# paramtres pour cet essai:
#
my $identifiant = 'pseudo-membre-boursorama';
my $motdepasse = 'mot-de-passe-boursorama';
#
my $libelle = "";
my $isin = "";
my $mnemo = "";
my $code;
my $ccount = 0;
my $lcount = 0;
my $date;
my ($jour,$mois,$an);
my $volume = "";
my $ouverture = "";
my $haut = "";
my $bas = "";
my $aujourdhui; # date du jour
my $hier;
my $avant_hier;
my $semaine_derniere;
#
# Rcuprer les options de la ligne de commande
#
our($opt_d);
our($opt_v);
if (getopts('d:v:')) {
    print $opt_d . "\n" if ($opt_d);
    print $opt_v . "\n" if ($opt_v);
} else {
    die "usage: autobull07 [-d aaaa-mm-jj][-v isin]\n";
}
# Gestion des cookies
my $cookie_jar = HTTP::Cookies->new();

# Cration du navigateur (user agent)
my $ua = LWP::UserAgent->new();
$ua->cookie_jar( $cookie_jar );

# On le fait passer pour Mozilla
$ua->agent("Mozilla/5.0 (X11; U; Linux i686; fr-FR; rv:1.7.2) Gecko/20040803");

#
# Visite de la page d'accueil
#
my $reponse = $ua->get("http://www.boursorama.com");
# Contrle de la rponse
if ($reponse->is_success) {
    $cookie_jar->extract_cookies($reponse); # rcupration des cookies
} else {
        die "Erreur accueil: " . $reponse->status_line . "\n";
}

# Authentification
#
my $requete = POST  'http://www.boursorama.com/logunique.phtml'  ;
my $contenu = "org=%2Findex.phtml%3F&redirect=&login=" . $identifiant . 
    "&password=" . $motdepasse;

$cookie_jar->add_cookie_header($requete) ;
$requete->content_type('application/x-www-form-urlencoded');
$requete->content_length(length($contenu));
$requete->content($contenu);
$reponse = $ua->request($requete) ;
# Contrle de la rponse
if ($reponse->is_error) {
        print "Erreur login: " . $reponse->status_line . "\n";
	exit(1);
}
$cookie_jar->extract_cookies($reponse); # rcupration des cookies
#
# Confirmer l'authentification par retour sur page index
#
$requete = HTTP::Request->new(GET => 'http://www.boursorama.com/index.phtml');
$cookie_jar->add_cookie_header($requete) ;
$reponse = $ua->request($requete);
# Contrle de la rponse
if ($reponse->is_success) {
    $cookie_jar->extract_cookies($reponse); # rcupration des cookies
} else {
        print "Erreur index: " . $reponse->status_line . "\n";
        exit(1);
}
#
# Ouvrir la base de donnes
#
my $dbh = DBI->connect('DBI:Pg:dbname=thebull',
		       'utilisateur-sql',
		       'mot-de-passe-sql',
		       {
			   AutoCommit => 0
		       }
		       ) or die "Erreur de connexion  la base: " . DBI->errstr;
#
# date de dbut de la mise  jour = dernire date enregistre dans les cotations
#
if ($opt_v) {
    $isin = $opt_v;
}
if ($opt_d) {
    $date = $opt_d;
} else {
    my $select_date = $dbh->prepare('SELECT MAX(jour) from cotations07');
    die "Impossible de prparer la requte de date\n"
	unless defined $select_date;
    $select_date->execute();
    if ($select_date->rows == 0) {
	die "Pas de cotation initiale\n";
    }
    $date = $select_date->fetchrow_array();
    $select_date->finish;
}
my $an1 = substr($date,0,4);
my $mois1 = substr($date,5,2);
my $jour1 = substr($date,8,2);
my $deltaliste;
#
# date de fin de la mise  jour = aujourd'hui
#
my $deltajours;
my @nomjs = ("Dimanche","Lundi","Mardi","Mercredi","Jeudi","Vendredi","Samedi");
my ($sec,$min,$heure,$jour2,$mois2,$an2,$js,$ja,$hiv) = localtime(time);
$an2 += 1900;
$mois2++;
$mois2 = $mois2 < 10 ? "0" . $mois2 : $mois2;
$jour2 = $jour2 < 10 ? "0" . $jour2 : $jour2;
#my $jour2 = 04;
#my $mois2 = 11;
#my $an2 = 2004;
$deltajours = Delta_Days($an2,$mois2,$jour2,$an1,$mois1,$jour1);
if (not $opt_d) {
    if (($js == 0) or ($js == 6)) {
	# le week-end on remonte jusqu' dix jours en arrire
	$deltaliste = -10;
    } else {
	# en semaine on remonte deux jours en arrire
	$deltaliste = -2;
    }
    if ($deltajours < $deltaliste) {
	$deltaliste = $deltajours;
    } else {
	$deltajours = $deltaliste;
    }
    ($an1,$mois1,$jour1) = Add_Delta_Days($an2,$mois2,$jour2,$deltaliste);
    $mois1 = $mois1 < 10 ? "0" . $mois1 : $mois1;
    $jour1 = $jour1 < 10 ? "0" . $jour1 : $jour1;
    $date = $an1 . "-" . $mois1 . "-" . $jour1;
}
if ($deltajours < -26) {
    $jour2 = $jour1;
    if ($mois1 < 12) {
	$mois2 = $mois1 + 1;
	$an2 = $an1;
    } else {
	$mois2 = 1;
	$an2 = $an1 + 1;
    }
    $deltaliste = Delta_Days($an2,$mois2,$jour2,$an1,$mois1,$jour1);
    if ($deltajours < $deltaliste) {
	$deltajours = $deltaliste;
    }
}
print "Du " . $date . "\n";
print "deltajours = " . $deltajours . "\n";
$aujourdhui = $an2 . "-" . $mois2 . "-" . $jour2;
print $nomjs[$js] . "\t" . $aujourdhui . "  " .$heure . "h" . $min . "\n";
($an,$mois,$jour) = Add_Delta_Days( Today(), -1 );
$mois = $mois < 10 ? "0" . $mois : $mois;
$jour = $jour < 10 ? "0" . $jour : $jour;
$hier = $an . "-" . $mois . "-" . $jour;
($an,$mois,$jour) = Add_Delta_Days( Today(), -2 );
$avant_hier = $an . "-" . $mois . "-" . $jour;
my ($an3,$mois3,$jour3) = Add_Delta_Days( Today(), -7 );
$semaine_derniere =  $an3 . "-" . $mois3 . "-" . $jour3;
#
# prparer l'insertion d'une cotation et sa recherche
#
my $insert_cotation = $dbh->prepare('INSERT INTO cotations07 (valeur,jour,cote) VALUES (?,?,?)');
my $select_cotation = $dbh->prepare('SELECT cote FROM cotations07 WHERE valeur = ? AND jour = ?');
my $delete_cotation = $dbh->prepare('DELETE FROM cotations07 WHERE valeur = ? AND jour = ?');
#
# prparer la liste des codes  tlcharger
# cotation = 0 -> code isin, cotation = -1 -> code mnemo simple
#
my $select_valeurs = $dbh->prepare('SELECT isin,valeur,libelle,mnemo,cotation FROM valeurs07 where cotation = 0 or cotation = -1');
#
# prparer la liste des valeurs spciales non disponibles en tlchargement
# cotation = code du pays (1=US, 33=FR) -> code isin
# cotation = -2 -> code mnemo avec prfixe et suffixe
#
my $select_valeurs_speciales = $dbh->prepare('SELECT isin,valeur,libelle,mnemo,cotation,prefixe,suffixe FROM valeurs07 where cotation > 0 or cotation = -2');
# prparer la recherche d'une valeur
my $select_valeur_isin = $dbh->prepare('SELECT valeur,libelle FROM valeurs07 WHERE isin = ?');
my $select_valeur_mnemo = $dbh->prepare('SELECT valeur,libelle FROM valeurs07 WHERE mnemo = ?');
my @valeur; # tableau de rsultat de la requte $select_valeur(s)
#
# prparer la mise  jour du libell pour chaque valeur
#
my $update_valeur = $dbh->prepare('UPDATE valeurs07 SET libelle = ? WHERE valeur = ?');
#
# prparer la recherche d'une valeur substitue  une valeur donne
#
my $select_substituee = $dbh->prepare('SELECT substitut,multiplicateur,diviseur FROM ost07 where valeur = ?');
#
# prparer la recherche d'une valeur de substitution
#
my $subcount;
my @substituee; # tableau de rsultat de la requte $select_substitution
my $select_substitution = $dbh->prepare('SELECT valeur,multiplicateur,diviseur,jour FROM ost07 where substitut = ?');
#
# prparer la recherche des valeurs substitues  une date donne
#
my $select_substituees = $dbh->prepare('SELECT DISTINCT valeur FROM ost07 where jour <= ?');
my $valsub; # rsultat de la requte select_substituees
#
# prparer la suppression des rsultats postrieurs  une date donne
#
my $delete_resultats = $dbh->prepare('DELETE FROM resultats07 WHERE jour >= ?');
my ($jourd,$moisd,$and) = (31,12,3000); # date de MAJ la plus ancienne

die "Impossible de prparer les requtes\n"
    unless defined $insert_cotation && defined $select_cotation 
        && defined $select_valeurs && defined $select_valeurs_speciales
        && defined $select_valeur_isin && defined $select_valeur_mnemo
        && defined $update_valeur
        && defined $select_substituee && defined $select_substitution
        && defined $select_substituees
        && defined $delete_resultats;
#
my $url;
my $buffer;
my $begin;
my $end;
my $cote;
my $pointeur;
my $dernier;
#
# slectionner les valeurs dont on peut tlcharger les cotations
#  partir du code isin ou du code mnmo
#
$select_valeurs->execute();
#
# pour chaque valeur de la liste obtenue on tlcharge les cours
#
while ( @valeur = $select_valeurs->fetchrow_array() ) {
#    last; # test debug
# exemple avec la premire valeur
#@valeur = $select_valeurs->fetchrow_array();
# exemple avec une valeur spcifie
#@valeur = ("FR0007049499",272,"Echiquier Ntor","",-10);
#@valeur = ("FR0010058644",367,"ORCHESTRA 1.55 BSA 03/09","KAZBS",-1);
#@valeur = ("US1509341079",368,"CELL THERAPEUTICS","CTIC",1);
#@valeur = ("FR0010166371",375,"ASSYSTEM BRIME BSAR 2012","ASBBS",0);
    if ($opt_v and ($valeur[0] ne $isin)) {
	next;
    }
    # filtrage des valeurs substitues
    $select_substituee->execute($valeur[1]);
    if ($select_substituee->rows > 0) {
	print "$valeur[2]: Valeur substitue\n";
	next;
    }
    # dtection des valeurs de substitution
    $select_substitution->execute($valeur[1]);
    $subcount = $select_substitution->rows;
    if ($subcount > 0) {
	$select_substitution->finish;
	print "$valeur[2]: Valeur de remplacement pour $subcount valeurs\n";
    }
    #
    # Aller sur la page de tlchargement chercher les donnes
    # (avec libell, date et cote de clture)
    #
    $url = URI->new('http://www.boursorama.com/outils/telechargement/telechargement.phtml');
    if ($valeur[4] == -1) {
	# code mnemo
	$code = $valeur[3];
    } else {
	# code isin
	$code = $valeur[0];
    }
    $url->query_form( 'hid_date' => 'ok', 'SELECT_PORT' => 'Tous',
		  'MARCHE' => 'SICOVAM', 'CODE' => $code,
		  'A_LIBELLE' => '1', 'A_DATE' => '1',
		  'A_CLOT' => '1', 'jour1' => $jour1, 'mois1' => $mois1,
		  'annee1' => $an1, 'jour2' => $jour2, 'mois2' => $mois2,
		  'annee2' => $an2, 'FILE_FORMAT' => 'LIBRE', 'ISINY' => 'Y',
		  'download' => 'T%E9l%E9charger' );
    $requete = HTTP::Request->new(GET => $url);
    $cookie_jar->add_cookie_header($requete) ;
    $reponse = $ua->request($requete);
    # Contrle de la rponse
    if ($reponse->is_error) {
        print "Erreur tele: " . $reponse->status_line . "\n";
        exit(1);
    }
    # rcupration de la rponse
    $buffer = $reponse->content;
    # exploitation des informations reues
    $end = index($buffer,"\t"); # 1re occurence de TAB en fin de libelle
    if ($end > 0) {
	# rcupration du libell de la valeur
	$libelle = substr($buffer,0,$end);
	print $libelle . "\n";
    }
    while($end > 0) {
	# rcupration de la date de cotation
	$jour = substr($buffer,$end + 1,2);
	if ((0+$jour == 0) or ($jour < 1) or ($jour > 31)) {
	    print "Erreur de date (jour=$jour) pour " . $valeur[0] . "\n";
	    last;
	}
	$mois = substr($buffer,$end + 4,2);
	if (($mois < 1) or ($mois > 12)) {
	    print "Erreur de date (mois=$mois) pour " . $valeur[0] . "\n";
	    last;
	}
	$an = substr($buffer,$end + 7,4);
	if (($an < 2003) or ($an > 2013)) {
	    print "Erreur de date (an=$an) pour " . $valeur[0] . "\n";
	    last;
	}
	$date = $an . "-" . $mois . "-" . $jour;
	# rcupration de la cote
	$begin = index($buffer,"\t",$end + 1); # 2nde occurence de TAB
	$end = index($buffer,"\n",$begin);
	$cote = substr($buffer,($begin + 1),($end - $begin - 1));
	if (defined($cote) and ($cote > 0)) {
	    # Est-ce qu'on l'a dj enregistre?
	    $select_cotation->execute($valeur[1],$date);
	    if ($select_cotation->rows == 0) {
		# enregistrement de la cotation de cette valeur  cette date
		$insert_cotation->execute($valeur[1],$date,$cote)
		    or die "Impossible d'ajouter la cote de " . $valeur[0] . "\n";
		print $date . "\t" . $cote . "\n";
		$ccount += 1;
		if (($an < $and) or (($an == $and) and ($mois < $moisd))
		    or (($an == $and) and ($mois == $moisd) and ($jour < $jourd))) {
		    print "$valeur[0]: $an-$mois-$jour avant $and-$moisd-$jourd\n";
		    $and = $an;
		    $moisd = $mois;
		    $jourd = $jour;
		}
		# suppression des cotes  cette date pour les valeurs substitues
		if ($subcount > 0) {
		    $select_substitution->execute($valeur[1]);
		    while ( @substituee = $select_substitution->fetchrow_array() ) {
			# Est-ce qu'on l'a dj enregistre?
			$select_cotation->execute($substituee[0],$date);
			if ($select_cotation->rows > 0) {
			    $delete_cotation->execute($substituee[0],$date)
				or die "Impossible de supprimer la cote de " . $substituee[0] . "  la date " . $date . "\n";
			}
		    }
		    $select_substitution->finish;
		}
	    }
	    $select_cotation->finish;
	} else {
	    print "Erreur de cote pour " . $valeur[0] . "\n";
	}
	$end = index($buffer,"\t",$end);
    }
    # dcompte des cotations
    if ($ccount > 0) {
	print $ccount . " mises  jour de cotation\n";
    }
    # mise  jour du libelle (si ncessaire)
    if (not defined($valeur[2])) {
	$update_valeur->execute($libelle,$valeur[1])
	    or die "Impossible d'ajouter le libell de " . $valeur[0] . "\n";
	$lcount = 1;
	print $lcount . " libell mis  jour\n";
    }
    if (($ccount > 0) or ($lcount > 0)) {
	$dbh->commit();
	$ccount = 0;
	$lcount = 0;
    }
}
$select_valeurs->finish;
$select_substituee->finish;
$select_substitution->finish;
#
# Aller sur la page de tlchargement chercher les donnes de la liste
# (avec isin, date et cote de clture)
#
#
#goto skiplist; # debug
$url = URI->new('http://www.boursorama.com/outils/telechargement/telechargement.phtml');
if (($js == 0) or ($js == 6)) {
    # le week-end on remonte jusqu' dix jours en arrire
    $deltaliste = -10;
} else {
    # en semaine on remonte deux jours en arrire
    $deltaliste = -2;
}
if ($deltajours < $deltaliste) {
    $deltaliste = $deltajours;
}
($an,$mois,$jour) = Add_Delta_Days($an2,$mois2,$jour2,$deltaliste);
$url->query_form( 'hid_date' => 'ok', 'SELECT_PORT' => 'Tous',
		  'MARCHE' => 'LISTE', 'CODE' => '',
		  'A_SICO' => '1', 'A_DATE' => '1',
		  'A_CLOT' => '1', 'jour1' => $jour, 'mois1' => $mois,
		  'annee1' => $an, 'jour2' => $jour2, 'mois2' => $mois2,
		  'annee2' => $an2, 'FILE_FORMAT' => 'LIBRE', 'ISINY' => 'Y',
		  'download' => 'T%E9l%E9charger' );
$requete = HTTP::Request->new(GET => $url);
$cookie_jar->add_cookie_header($requete) ;
$reponse = $ua->request($requete);
# Contrle de la rponse
if ($reponse->is_error) {
    print "Erreur tele: " . $reponse->status_line . "\n";
    exit(1);
}
# rcupration de la rponse
$buffer = $reponse->content;
# exploitation des informations reues
$begin = 0;
$end = index($buffer,"\t"); # 1re occurence de TAB en fin de code isin
$isin = 'INCONNU';
$valeur[0] = 0;
while($end > 0) {
    # rcupration du code isin de la valeur
    if ($isin ne substr($buffer,$begin,$end - $begin)) {
	$isin = substr($buffer,$begin,$end - $begin);
	print $isin . "\t";
	$select_valeur_isin->execute($isin);
	if ($select_valeur_isin->rows == 0) {
	    $select_valeur_isin->finish;
	    $select_valeur_mnemo->execute($isin);
	    if ($select_valeur_mnemo->rows == 0) {
		$select_valeur_mnemo->finish;
		print "valeur inutilise\n";
		$valeur[0] = 0;
	    } else {
		@valeur = $select_valeur_mnemo->fetchrow_array();
		print $valeur[1] . "\n";
	    }
	} else {
	    @valeur = $select_valeur_isin->fetchrow_array();
	    print $valeur[1] . "\n";
	}
    }
    if ($valeur[0] == 0) {
        # sauter l'inconnu
	$begin = index($buffer,"\n",$end + 1) + 1;
	$end = index($buffer,"\t",$begin);
	next;
    }
    # rcupration de la date de cotation
    $jour = substr($buffer,$end + 1,2);
    $mois = substr($buffer,$end + 4,2);
    $an = substr($buffer,$end + 7,4);
    $date = $an . "-" . $mois . "-" . $jour;
    # rcupration de la cote
    $begin = index($buffer,"\t",$end + 1); # 2nde occurence de TAB
    $end = index($buffer,"\n",$begin);
    $cote = substr($buffer,($begin + 1),($end - $begin - 1));
    if (defined($cote) and ($cote > 0)) {
        # Est-ce qu'on l'a dj enregistre?
        $select_cotation->execute($valeur[0],$date);
	if ($select_cotation->rows == 0) {
	    # enregistrement de la cotation de cette valeur  cette date
	    $insert_cotation->execute($valeur[0],$date,$cote)
		or die "Impossible d'ajouter la cote de " . $isin . "\n";
	    print $date . "\t" . $cote . "\n";
	    $ccount += 1;
	    if (($an < $and) or (($an == $and) and ($mois < $moisd))
		or (($an == $and) and ($mois == $moisd) and ($jour < $jourd))) {
		print "$valeur[0]: $an-$mois-$jour avant $and-$moisd-$jourd\n";
		$and = $an;
		$moisd = $mois;
		$jourd = $jour;
	    }
	}
	$select_cotation->finish;
    } else {
	print "Erreur de cote pour " . $isin . "\n";
    }
    $begin = $end + 1; # on se positionne sur la ligne suivante (aprs le "\n")
    $end = index($buffer,"\t",$begin);
}
$select_valeur_isin->finish;
$select_valeur_mnemo->finish;
#
# Pour les valeurs qu'on ne peut pas avoir en tlchargement
#
skiplist:
$select_valeurs_speciales->execute();
while ( @valeur = $select_valeurs_speciales->fetchrow_array() ) {
#@valeur = ("FR0004152593",166,"A NOVO","NOV",0);
#@valeur = ("FR0010058644",367,"ORCHESTRA 1.55 BSA 03/09","KAZBS",33);
#@valeur = ("US1509341079","368","CELL THERAPEUTICS","CTIC",1);
    if ($opt_v and ($valeur[0] ne $isin)) {
	next;
    }
    print $valeur[0] . "\t" . $valeur[2];
    #
    # Aller sur la page de recherche de cours avec le code isin
    #
    if ($valeur[4] != -2) {
	$url = URI->new('http://www.boursorama.com/cours.phtml');
	$url->query_form( 'code' => $valeur[0],
    		  'choix_bourse' => "pays=$valeur[4]",
    		  'categorie' => '' );
    } else {
	$url = URI->new('http://www.boursorama.com/cours.phtml');
	$url->query_form( 'symbole' => $valeur[5] . $valeur[3] . $valeur[6]);
	# Remettre un code pays pour ces valeurs de cotation=-2
	if (($valeur[5] eq '') or ($valeur[5] eq '3k')) {
	    # prefixe absent=>NYSE, prefixe=3k=>NASDAQ
	    $valeur[4] = 1; # Etats-Unis
	} elsif ($valeur[5] eq '3t') {
	    # prefixe=3t=>Toronto
	    $valeur[4] = 2; # Canada
	} elsif ($valeur[5] eq '3eF') {
	    # prefixe=3eF=>Francfort
	    $valeur[4] = 49; # Allemagne
	} elsif ($valeur[5] eq '1u') {
	    # prefixe=1u=>Londres
	    $valeur[4] = 44; # Angleterre
	}
    }
    $requete = HTTP::Request->new(GET => $url);
    $cookie_jar->add_cookie_header($requete) ;
    $reponse = $ua->request($requete);
    # Contrle de la rponse
    if ($reponse->is_error) {
	print "Erreur cours: " . $reponse->status_line . "\n";
	exit(1);
    }
    # rcupration de la rponse
    $buffer = $reponse->content;
    # exploitation des informations reues
    if ($buffer =~ /<TITLE>Boursorama.+Recherche.+valeur<\/TITLE>/) {
	#
	# la valeur n'est pas trouve avec le code isin
	# essayer le code mnmonique
	#
	$url->query_form( 'code' => $valeur[3],
    		  'choix_bourse' => "pays=$valeur[4]",
    		  'categorie' => '' );
	$requete = HTTP::Request->new(GET => $url);
	$cookie_jar->add_cookie_header($requete) ;
	$reponse = $ua->request($requete);
	# Contrle de la rponse
	if ($reponse->is_error) {
	    print "Erreur cours: " . $reponse->status_line . "\n";
	    exit(1);
	}
	# rcupration de la rponse
	$buffer = $reponse->content;
	# exploitation des informations reues
	if ($buffer =~ /<TITLE>Boursorama.+Recherche.+valeur<\/TITLE>/) {
	    #
	    # la valeur n'est pas trouve
	    #
	}
    }
#    if ($buffer =~ m/<TITLE>(.*?) : /) { # valable jusqu'au 15/4/2005
    if ($buffer =~ m/<TITLE>Cours Action (.*?) - /) { # valable depuis le 15/4/2005
	#
	# la valeur est trouve dans les actions
	#   extraire le libell, le code isin, la dernire cote et le volume
	#
	$date = '';
	$libelle = $1;
	$pointeur = index($buffer," - Info - Cotation - ") + 21; # valable depuis le 5/12/2005
	if ($pointeur > 0) {
	    $isin = substr($buffer,$pointeur,12);
	} else {
	    $isin = '';
	}
#	print "isin=" . $isin . "\n"; #debug
	$dernier = index($buffer,"<!-- AUTRES LIENS VALEUR -->");
	if ($dernier > 0) {
	    $pointeur = index($buffer,"<TD colspan=2 align=left><font size=\"+1\"><b>") + 44; # valable depuis le 6/12/2006
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $dernier = index($buffer,"<",$pointeur);
	    $libelle = substr($buffer,$pointeur,$dernier - $pointeur);
	} else {
	    $libelle = '';
	}
#	print "libelle=" . $libelle . "\n"; #debug
	$dernier = index($buffer,"width=48% class=\"ligneCours\" align=\"right\" nowrap><span class=\"p2b\">"); # valable depuis le 12/2/2007
	if ($dernier > 0) {
	    $pointeur = index($buffer,"\"p2b\">",$dernier) + 6;
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $cote = substr($buffer,$pointeur,8);
#	    print "cote brute=" . $cote . "\n"; #debug
	    $cote =~ s/ //; # recolle le chiffre des milliers
	    if ($cote =~ /([0-9\.]+)/) {
		$cote = $1;
	    }
	} else {
	    $cote = '';
	}
#	print "cote=" . $cote . "\n"; #debug
	$dernier = index($buffer,"Volume") + 12; # valable depuis le 12/2/2007
	if ($dernier > 0) {
	    $pointeur = index($buffer,"\"right\">",$dernier) + 8;
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $volume = substr($buffer,$pointeur,12);
	    $volume =~ s/ //; # recolle le chiffre des milliers et des millions
	    if ($volume =~ /([0-9\ ]+)/) {
		$volume = $1;
	    }
	} else {
	    $volume = '';
	}
#	print "volume=" . $volume . "\n"; #debug
	$dernier = index($buffer,"Ouverture") + 12; # valable depuis le 12/2/2007
	if ($dernier > 0) {
	    $pointeur = index($buffer,"\"right\">",$dernier) + 8;
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $ouverture = substr($buffer,$pointeur,12);
	    if ($ouverture =~ /([0-9\.]+)/) {
		$ouverture = $1;
	    }
	} else {
	    $ouverture = '';
	}
#	print "ouverture=" . $ouverture . "\n"; #debug
	$dernier = index($buffer,"+ Haut") + 12; # valable depuis le 12/2/2007
	if ($dernier > 0) {
	    $pointeur = index($buffer,"\"right\">",$dernier) + 8;
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $haut = substr($buffer,$pointeur,12);
	    if ($haut =~ /([0-9\.]+)/) {
		$haut = $1;
	    }
	} else {
	    $haut = '';
	}
#	print "haut=" . $haut . "\n"; #debug
	$dernier = index($buffer,"+ Bas") + 12; # valable depuis le 12/2/2007
	if ($dernier > 0) {
	    $pointeur = index($buffer,"\"right\">",$dernier) + 8;
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $bas = substr($buffer,$pointeur,12);
	    if ($bas =~ /([0-9\.]+)/) {
		$bas = $1;
	    }
	} else {
	    $bas = '';
	}
#	print "bas=" . $bas . "\n"; #debug
    } elsif ($buffer =~ m/<TITLE>(.*?) - /) {
	#
	# la valeur est trouve dans les FCP
	#   extraire le libell, le code isin et la dernire cote
	#
	$volume = '';
	$libelle = $1;
	$pointeur = index($buffer,"\"TopPage\">");
	if ($pointeur > 0) {
	    $isin = substr($buffer,$pointeur + 27 + length($libelle),12);
	} else {
	    $isin = '';
	}
	$dernier = index($buffer,"Valeur liquidative</td>");
	if ($dernier > 0) {
	    $pointeur = index($buffer,"right><B>",$dernier + 20);
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    $cote = substr($buffer,$pointeur + 9,8);
	    if ($cote =~ /([0-9\.]+)/) {
		$cote = $1;
	    }
	} else {
	    $cote = '';
	}
	$dernier = index($buffer,"Date",$dernier);
	if ($dernier > 0) {
	    $pointeur = index($buffer,"right><B>",$dernier + 20);
	} else {
	    $pointeur = -1;
	}
	if ($pointeur > 0) {
	    # rcupration de la date au format an-mois-jour
	    $jour = substr($buffer,$pointeur + 9,2);
	    $mois = substr($buffer,$pointeur + 12,2);
	    $an = substr($buffer,$pointeur + 15,4);
	    $date = $an . "-" . $mois . "-" . $jour;
	} else {
	    $date = "";
	}
    }
    if (($isin eq $valeur[0]) or ($libelle eq $valeur[2]) or ($libelle eq $valeur[3])) {
#	print "heure=" . $heure . "\n"; # debug
	if ($volume and (($ouverture > 0) or ($haut > 0) or ($bas > 0))) {
	    # contrle de l'heure
	    if ($js == 0) {
		$date = $avant_hier;
	    } elsif ($js == 6) {
		$date = $hier;
	    } elsif (($valeur[4] == 1) or ($valeur[4] == 2)) {
		# US et Canada cotations entre 15h30 et 23h
		if (($heure < 15) or (($heure == 15) and ($min < 30))) {
		    $date = $hier;
		} elsif ($heure > 22) {
		    $date = $aujourdhui;
		}
	    } else {
		# Europe cotations entre 8h et 17h35
		if ($heure < 9) {
		    $date = $hier;
		} elsif ((($heure == 17) and ($min > 35)) or ($heure > 17)) {
		    $date = $aujourdhui;
		}
	    }
	}
#	print "date=" . $date . "\n"; # debug
        if ($date) {
	    # Est-ce qu'on l'a dj enregistre?
	    $select_cotation->execute($valeur[1],$date);
	    if ($select_cotation->rows == 0) {
		# enregistrement de la cotation de cette valeur  cette date
		$insert_cotation->execute($valeur[1],$date,$cote)
		    or die "Impossible d'ajouter la cote de " . $valeur[0] . "\n";
		print "\t" . $date . "\t" . $cote . "\n";
		$an = substr($date,0,4);
		$mois = substr($date,5,2);
		$jour = substr($date,8,2);
		$ccount += 1;
		if (($an < $and) or (($an == $and) and ($mois < $moisd))
		    or (($an == $and) and ($mois == $moisd) and ($jour < $jourd))) {
		    print "$valeur[0]: $an-$mois-$jour avant $and-$moisd-$jourd\n";
		    $and = $an;
		    $moisd = $mois;
		    $jourd = $jour;
		}
	    } else {
		print "\n";
	    }
	    $select_cotation->finish;
	} else {
	    print "\n";
	}
    } else {
	print "\tnon trouve\n";
    }
#    last; # debug test sur la premire valeur seulement
}
$select_valeurs_speciales->finish;
#
# Mise  jour des cotations des valeurs substitues
#   partir de la date de dbut de la collecte ($an1,$mois1,$jour1)
#  et jusqu' la date de fin de la collecte ($an2,$mois2,$jour2)
#
($an,$mois,$jour) = ($an1,$mois1,$jour1);
while (Delta_Days($an,$mois,$jour,$an2,$mois2,$jour2) > 0) {
    $date = $an . "-" . $mois . "-" . $jour;
    print $date . "\n"; # debug
    # slection des valeurs substitues  cette date
    $select_substituees->execute($date);
    while ( $valsub = $select_substituees->fetchrow_array() ) {
	print "valsub = " . $valsub . "\n"; # debug
	# Est-ce qu'on l'a dj enregistre?
	$select_cotation->execute($valsub,$date);
	if ($select_cotation->rows == 0) {
	    $cote = 0;
	    my $manque = 0;
	    $select_substituee->execute($valsub);
	    while ( @substituee = $select_substituee->fetchrow_array() ) {
		# chercher la cote de la valeur de substitution
		$select_cotation->execute($substituee[0],$date);
		if ($select_cotation->rows == 0) {
		    $manque = 1; # pas de calcul possible s'il manque une cote
		    print "cote manquante pour " . $substituee[0] . " au " . $date . "\n";
		    last;
		}
		my $subcote = $select_cotation->fetchrow_array();
		$cote = $cote + (($subcote * $substituee[1])/$substituee[2]);
	    }
	    $select_substituee->finish;
	    if (!$manque) {
		$cote = sprintf("%.2f", $cote);
		$insert_cotation->execute($valsub,$date,$cote)
		    or die "Impossible d'ajouter la cote de " . $valsub . "\n";
		print "sub = " . $valsub . "\t" . $cote . "\n";
		$ccount += 1;
		if (($an < $and) or (($an == $and) and ($mois < $moisd))
		    or (($an == $and) and ($mois == $moisd) and ($jour < $jourd))) {
		    print "$valsub: $an-$mois-$jour avant $and-$moisd-$jourd\n";
		    $and = $an;
		    $moisd = $mois;
		    $jourd = $jour;
		}
	    }
	}
    }
    ($an,$mois,$jour) = Add_Delta_Days( $an, $mois, $jour, 1 );
}
$select_substituees->finish;
$select_cotation->finish;
# dcompte des cotations
if ($ccount > 0) {
    $dbh->commit();
    print "commit\n"; # debug
    print $ccount . " mises  jour de cotation\n";
}
#
# suppression des rsultats concerns par les cotations mises  jour
#
if ($and < 3000) {
    $date = $and . "-" . $moisd . "-" . $jourd;
    $delete_resultats->execute($date);
    $ccount = $delete_resultats->rows;
#    $ccount = 0; # debug
    print $ccount . " suppressions de rsultats  compter du " . $date . "\n";
}
if ($ccount > 0) {
    $dbh->commit();
    $ccount = 0;
}
#
# fin de la recherche, on ferme!
#
$dbh->disconnect;

#
# Logout
#
$requete= HTTP::Request->new(GET => 'http://www.boursorama.com/deconnexion.phtml');
$cookie_jar->add_cookie_header($requete) ;
$reponse = $ua->request($requete);
# Contrle de la rponse
if ($reponse->is_success) {
} else {
        print "Erreur logout: " . $reponse->status_line . "\n";
        exit(1);
}

exit(0);
